# -*- coding: utf-8 -*-

'''
Functions for processing Japanese text

Latest version can be found at https://github.com/letuananh/chirptext

References:
    MeCab homepage:
        http://taku910.github.io/mecab/

MeCab, デコ, got the joke?
* This script is used to be a part of omwtk

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2017, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import re
import os
import logging

import MeCab
import jaconv


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

def getLogger():
    return logging.getLogger(__name__)


DATA_FOLDER = os.path.abspath(os.path.expanduser('./data'))

# reference: https://en.wikipedia.org/wiki/Hiragana_%28Unicode_block%29
# U+304x 		ぁ 	あ 	ぃ 	い 	ぅ 	う 	ぇ 	え 	ぉ 	お 	か 	が 	き 	ぎ 	く
# U+305x 	ぐ 	け 	げ 	こ 	ご 	さ 	ざ 	し 	じ 	す 	ず 	せ 	ぜ 	そ 	ぞ 	た
# U+306x 	だ 	ち 	ぢ 	っ 	つ 	づ 	て 	で 	と 	ど 	な 	に 	ぬ 	ね 	の 	は
# U+307x 	ば 	ぱ 	ひ 	び 	ぴ 	ふ 	ぶ 	ぷ 	へ 	べ 	ぺ 	ほ 	ぼ 	ぽ 	ま 	み
# U+308x 	む 	め 	も 	ゃ 	や 	ゅ 	ゆ 	ょ 	よ 	ら 	り 	る 	れ 	ろ 	ゎ 	わ
# U+309x 	ゐ 	ゑ 	を 	ん 	ゔ 	ゕ 	ゖ 			゙ 	゚ 	゛ 	゜ 	ゝ 	ゞ 	ゟ

HIRAGANA = 'ぁあぃいぅうぇえぉおかがきぎくぐけげこごさざしじすずせぜそぞただちぢっつづてでとどなにぬねのはばぱひびぴふぶぷへべぺほぼぽまみむめもゃやゅゆょよらりるれろゎわゐゑをんゔゕゖ゙゚゛゜ゝゞゟ'

# reference: https://en.wikipedia.org/wiki/Katakana_%28Unicode_block%29
# U+30Ax 	゠ 	ァ 	ア 	ィ 	イ 	ゥ 	ウ 	ェ 	エ 	ォ 	オ 	カ 	ガ 	キ 	ギ 	ク
# U+30Bx 	グ 	ケ 	ゲ 	コ 	ゴ 	サ 	ザ 	シ 	ジ 	ス 	ズ 	セ 	ゼ 	ソ 	ゾ 	タ
# U+30Cx 	ダ 	チ 	ヂ 	ッ 	ツ 	ヅ 	テ 	デ 	ト 	ド 	ナ 	ニ 	ヌ 	ネ 	ノ 	ハ
# U+30Dx 	バ 	パ 	ヒ 	ビ 	ピ 	フ 	ブ 	プ 	ヘ 	ベ 	ペ 	ホ 	ボ 	ポ 	マ 	ミ
# U+30Ex 	ム 	メ 	モ 	ャ 	ヤ 	ュ 	ユ 	ョ 	ヨ 	ラ 	リ 	ル 	レ 	ロ 	ヮ 	ワ
# U+30Fx 	ヰ 	ヱ 	ヲ 	ン 	ヴ 	ヵ 	ヶ 	ヷ 	ヸ 	ヹ 	ヺ 	・ 	ー 	ヽ 	ヾ 	ヿ
KATAKANA = '゠ァアィイゥウェエォオカガキギクグケゲコゴサザシジスズセゼソゾタダチヂッツヅテデトドナニヌネノハバパヒビピフブプヘベペホボポマミムメモャヤュユョヨラリルレロヮワヰヱヲンヴヵヶヷヸヹヺ・ーヽヾヿ'


# -------------------------------------------------------------------------------
# Data Structures
# -------------------------------------------------------------------------------

# Reference: http://taku910.github.io/mecab/#parse
# MeCabToken = namedtuple('MeCabToken', 'surface pos sc1 sc2 sc3 inf conj root reading pron'.split())
class MeCabToken(object):
    def __init__(self, surface, pos=None, sc1=None, sc2=None, sc3=None, inf=None, conj=None, root=None, reading=None, pron=None):
        self.surface = surface
        self.pos = pos
        self.sc1 = sc1
        self.sc2 = sc2
        self.sc3 = sc3
        self.inf = inf
        self.conj = conj
        self.root = root
        self.reading = reading
        self.pron = pron
        self.is_eos = (surface == 'EOS' and pos == sc1 == sc2 == sc3 == inf == conj == root == reading == pron == '')

    def __str__(self):
        return '[{sur}({pos}-{s1}/{s2}/{s3}|{ro}|{re}|{pr})]'.format(sur=self.surface, pos=self.pos, s1=self.sc1, s2=self.sc2, s3=self.sc3, ro=self.root, re=self.reading, pr=self.pron)

    def __repr__(self):
        return str(self)

    def reading_hira(self):
        return jaconv.kata2hira(self.reading)

    def need_ruby(self):
        return self.reading and self.reading != self.surface and self.reading_hira() != self.surface

    def pos3(self):
        ''' Use pos-sc1-sc2 as POS '''
        parts = [self.pos]
        if self.sc1 and self.sc1 != '*':
            parts.append(self.sc1)
            if self.sc2 and self.sc2 != '*':
                parts.append(self.sc2)
        return '-'.join(parts)

    def to_ruby(self):
        ''' Convert one MeCabToken into HTML '''
        if self.need_ruby():
            surface = self.surface
            reading = self.reading_hira()
            return '<ruby><rb>{sur}</rb><rt>{read}</rt></ruby>'.format(sur=surface, read=reading)
        elif self.is_eos:
            return ''
        else:
            return self.surface

    @staticmethod
    def parse(raw):
        parts = re.split('\t|,', raw)
        if len(parts) < 10:
            parts += [''] * (10 - len(parts))
        (surface, pos, sc1, sc2, sc3, inf, conj, root, reading, pron) = parts
        return MeCabToken(surface, pos, sc1, sc2, sc3, inf, conj, root, reading, pron)

    def to_csv(self):
        return '{sur}\t{pos}\t{s1}\t{s2}\t{s3}\t{inf}\t{conj}\t{ro}\t{re}\t{pr}'.format(sur=self.surface, pos=self.pos, s1=self.sc1, s2=self.sc2, s3=self.sc3, inf=self.inf, conj=self.conj, ro=self.root, re=self.reading, pr=self.pron)


class MeCabSent(object):

    mecab = MeCab.Tagger()

    def __init__(self, surface, tokens):
        self.surface = surface
        self.tokens = []
        if tokens:
            self.tokens.extend(tokens)

    def to_ruby(self):
        s = []
        for x in self.tokens:
            s.append(x.to_ruby())
        stext = ' '.join(s)
        # clean sentence a bit ...
        stext = stext.replace(' 。', '。').replace('「 ', '「').replace(' 」', '」').replace(' 、 ', '、').replace('（ ', '（').replace(' ）', '）')
        return stext

    def __getitem__(self, value):
        return self.tokens[value]

    def __len__(self):
        return len(self.tokens)

    @property
    def words(self):
        return [x.surface for x in self.tokens if not x.is_eos]

    def __str__(self):
        return ' '.join([x.surface for x in self.tokens if not x.is_eos])

    @staticmethod
    def parse(text):
        ''' Use mecab to parse one sentence '''
        mecab_out = MeCabSent.mecab.parse(text).splitlines()
        tokens = [MeCabToken.parse(x) for x in mecab_out]
        return MeCabSent(text, tokens)


class DekoText(object):

    def __init__(self):
        self.sents = []

    def __len__(self):
        return len(self.sents)

    def __getitem__(self, name):
        return self.sents[name]

    def add(self, sentence_text):
        ''' Parse a text string and add it to this doc '''
        sent = MeCabSent.parse(sentence_text)
        self.sents.append(sent)
        return sent

    def __str__(self):
        return "\n".join(["#{}. {}".format(idx + 1, x) for idx, x in enumerate(self)])

    @staticmethod
    def parse(text, splitlines=True, auto_strip=True):
        doc = DekoText()
        if not splitlines:
            # surface is broken right now ...
            tokens = txt2mecab(text)
            doc.sents = tokenize_sent(tokens, text, auto_strip)
        else:
            lines = text.splitlines()
            for line in lines:
                if auto_strip:
                    doc.add(line.strip())
                else:
                    doc.add(line)
        return doc


# -------------------------------------------------------------------------------
# Functions
# -------------------------------------------------------------------------------

def txt2mecab(text):
    ''' Use mecab to parse one sentence '''
    mecab = MeCab.Tagger()
    mecab_out = mecab.parse(text).splitlines()
    tokens = [MeCabToken.parse(x) for x in mecab_out]
    return MeCabSent(text, tokens)


def lines2mecab(lines):
    ''' Use mecab to parse many lines '''
    sents = []
    for line in lines:
        sent = txt2mecab(line)
        sents.append(sent)
    return sents


# TODO: Need to calculate cfrom, cto to get surfaces
def tokenize_sent(mtokens, raw='', auto_strip=True):
    sents = []
    bucket = []
    cfrom = 0
    cto = 0
    token_cfrom = 0
    for t in mtokens:
        if t.is_eos:
            break
        token_cfrom = raw.find(t.surface, cto)
        cto = token_cfrom + len(t.surface)
        if not t.is_eos:
            bucket.append(t)
        if t.pos == '記号' and t.sc1 == '句点':
            sent_text = raw[cfrom:cto]
            if auto_strip:
                sent_text = sent_text.strip()
            sents.append(MeCabSent(sent_text, bucket))
            cfrom = cto
            bucket = []
    if bucket:
        sent_text = raw[cfrom:cto]
        if auto_strip:
            sent_text = sent_text.strip()
        sents.append(MeCabSent(sent_text, bucket))
    return sents


def wakati(content):
    return MeCab.Tagger("-O wakati").parse(content)


def tokenize(content):
    ''' Sentence to a list of tokens (string) '''
    # TODO: Check if wakati better?
    # return wakati(content).split(' ')
    return txt2mecab(content).words


def analyse(content, splitlines=True, format=None):
    ''' Japanese text > tokenize/txt/html '''
    sents = DekoText.parse(content, splitlines=splitlines)
    doc = []
    final = sents
    # Generate output
    if format == 'html':
        for sent in sents:
            doc.append(sent.to_ruby())
        final = '<br/>\n'.join(doc)
    elif format == 'csv':
        for sent in sents:
            doc.append('\n'.join([x.to_csv() for x in sent]) + '\n')
        final = '\n'.join(doc)
    elif format == 'txt':
        final = '\n'.join([str(x) for x in sents])
    return final



# -*- coding: utf-8 -*-

''' Text Annotation library

Latest version can be found at https://github.com/letuananh/chirptext

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2014, Le Tuan Anh <tuananh.ke@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
import logging
import json
import csv
from collections import namedtuple
from collections import defaultdict as dd
from collections import OrderedDict

from chirptext import FileHelper, FileHub
from chirptext.io import CSV


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

OPEN_TAG = "<wnsk>"
CLOSE_TAG = "</wnsk>"
STD_DIALECT = 'excel-tab'
STD_QUOTING = csv.QUOTE_MINIMAL
TokenInfo = namedtuple("TokenInfo", ['text', 'sk'])


def getLogger():
    return logging.getLogger(__name__)


# ------------------------------------------------------------------------------
# Classes
# ------------------------------------------------------------------------------

class TagInfo(object):

    GOLD = 'gold'
    ISF = 'isf'
    MFS = 'mfs'
    LELESK = 'lelesk'
    OTHER = 'other'
    NTUMC = 'ntumc'
    MECAB = 'mecab'
    NLTK = 'nltk'
    DEFAULT = 'n/a'

    def __init__(self, cfrom=-1, cto=-1, label='', source=DEFAULT, tagtype=''):
        self.cfrom = cfrom
        self.cto = cto
        self.label = label
        self.source = source
        self.tagtype = tagtype  # tag type

    def __repr__(self):
        if not self.tagtype:
            return '`{}`'.format(self.label)
        else:
            return '`{}:{}`'.format(self.tagtype, self.label)

    def __str__(self):
        return "`{l}`<{f}:{t}>{v}".format(l=self.label, f=self.cfrom, t=self.cto, v=self.tagtype)


class TaggedSentence(object):

    def __init__(self, text, tags=None, tokens=None, ID=None):
        self.text = text
        self.__tags = list(tags) if tags else []
        self._tokens = tokens if tokens else []
        self.concept_map = OrderedDict()  # concept.cid to concept object
        self.ID = ID

    def __str__(self):
        # return format_tag(self)
        if self.ID:
            return "#{id}: {txt}".format(id=self.ID, txt=self.text)
        else:
            return self.text

    def __getitem__(self, idx):
        return self._tokens[idx]

    def __len__(self):
        return len(self._tokens)

    @property
    def tags(self):
        ''' Sentence level tags '''
        return self.__tags

    @property
    def tokens(self):
        return self._tokens

    @property
    def concepts(self):
        return tuple(self.concept_map.values())

    @property
    def wclinks(self):
        ''' Return word-concepts map '''
        wcm = dd(list)
        for concept in self.concept_map.values():
            for w in concept.words:
                wcm[w].append(concept)
        return wcm

    @property
    def mwe(self):
        ''' Get all multi-word expressions '''
        mwe = []
        for c in self.concept_map.values():
            if len(c.words) > 1:
                mwe.append(c)
        return mwe

    @property
    def msw(self):
        ''' Get words that were tagged with multiple senses '''
        return [w for w, c in self.wclinks.items() if len(c) > 1]

    def surface(self, tag):
        if tag.cfrom >= 0 and tag.cto >= 0:
            return self.text[tag.cfrom:tag.cto]
        else:
            return ''

    def add_tag(self, label, cfrom=-1, cto=-1, source=TagInfo.DEFAULT, tagtype=''):
        ''' Add a sentence-level tag '''
        self.tags.append(TagInfo(cfrom, cto, label=label, source=source, tagtype=tagtype))

    def add_token(self, label, cfrom=-1, cto=-1, source=TagInfo.DEFAULT):
        tk = Token(cfrom, cto, label, self)
        self._tokens.append(tk)
        return tk

    def import_tokens(self, tokens, import_hook=None, ignorecase=True):
        text = self.text.lower() if ignorecase else self.text
        has_hooker = import_hook and callable(import_hook)
        cfrom = 0
        for token in tokens:
            if has_hooker:
                import_hook(token)
            start = text.find(token.lower() if ignorecase else token, cfrom)
            if start == -1:
                raise LookupError('Cannot find token `{t}` in sent `{s}`({l}) from {i} ({p})'.format(t=token, s=self.text, l=len(self.text), i=cfrom, p=self.text[cfrom:cfrom + 20]))
            cfrom = start
            cto = cfrom + len(token)
            self.add_token(token, cfrom, cto)
            cfrom = cto - 1

    def fix_cfrom_cto(self, import_hook=None, ignorecase=True):
        text = self.text.lower() if ignorecase else self.text
        has_hooker = import_hook and callable(import_hook)
        cfrom = 0
        for token in self.tokens:
            if has_hooker:
                import_hook(token.label)
            start = text.find(token.label.lower() if ignorecase else token.label, cfrom)
            if start == -1:
                raise LookupError('Cannot find token `{t}` in sent `{s}`({l}) from {i} ({p})'.format(t=token, s=self.text, l=len(self.text), i=cfrom, p=self.text[cfrom:cfrom + 20]))
            cfrom = start
            cto = cfrom + len(token.label)
            token.cfrom = cfrom
            token.cto = cto
            cfrom = cto - 1

    def add_concept(self, cid=None, clemma="", tag="", words=None):
        ''' Add a new concept object '''
        if cid is None:
            cid = self.new_concept_id()
        c = Concept(cid, clemma, tag, self, list(words) if words is not None else None)
        self.concept_map[cid] = c
        return c

    def new_concept_id(self):
        cid = 0
        while cid in self.concept_map:
            cid += 1
        return cid

    def tag(self, clemma, tag, *word_ids):
        ''' Tag word(s) with new concept '''
        cid = self.new_concept_id()
        return self.add_concept(cid, clemma, tag, [self[x] for x in word_ids])

    def concept(self, cid):
        ''' Get a concept by concept ID '''
        return self.concept_map[cid]

    def to_json(self):
        sent_dict = {'text': self.text,
                     'tokens': [t.to_json() for t in self.tokens],
                     'concepts': [c.to_json() for c in self.concepts]}
        if self.ID is not None:
            sent_dict['ID'] = self.ID
        return sent_dict

    @staticmethod
    def from_json(json_sent):
        sent = TaggedSentence(json_sent['text'])
        if 'ID' in json_sent:
            sent.ID = json_sent['ID']
        for json_token in json_sent['tokens']:
            token = sent.add_token(json_token['label'], json_token['cfrom'], json_token['cto'])
            if Token.POS in json_token:
                token.tag(json_token[Token.POS], tagtype=Token.POS)
            if Token.LEMMA in json_token:
                token.tag(json_token[Token.LEMMA], tagtype=Token.LEMMA)
            if Token.COMMENT in json_token:
                token.tag(json_token[Token.COMMENT], tagtype=Token.COMMENT)
        # import concepts
        for json_concept in json_sent['concepts']:
            clemma = json_concept['clemma']
            tag = json_concept['tag']
            wordids = json_concept['words']
            concept = sent.tag(clemma, tag, *wordids)
            if Concept.COMMENT in json_concept:
                concept.comment = json_concept[Concept.COMMENT]
            if Concept.FLAG in json_concept:
                concept.flag = json_concept[Concept.FLAG]

        return sent


class Token(object):

    LEMMA = 'lemma'
    POS = 'pos'
    COMMENT = 'comment'

    def __init__(self, cfrom, cto, label, sent=None, tags=None, source=TagInfo.DEFAULT):
        ''' A token (e.g. a word in a sentence) '''
        self.cfrom = cfrom
        self.cto = cto
        self._tags = tags if tags else []
        self.sent = sent
        self.label = label

    def __getitem__(self, idx):
        return self._tags[idx]

    def __len__(self):
        return len(self._tags)

    @property
    def surface(self):
        if self.sent and self.sent.text:
            return self.sent.text[self.cfrom:self.cto]
        else:
            return ''

    @property
    def pos(self):
        tm = self.tag_map
        if self.POS in tm:
            return tm[self.POS][0].label
        else:
            return ''

    @property
    def lemma(self):
        tm = self.tag_map
        if self.LEMMA in tm:
            return tm[self.LEMMA][0].label
        else:
            return ''

    @property
    def comment(self):
        tm = self.tag_map
        if Token.COMMENT in tm:
            return tm[self.COMMENT][0].label
        else:
            return ''

    def __repr__(self):
        return "`{l}`<{f}:{t}>".format(l=self.label, f=self.cfrom, t=self.cto)

    @property
    def tag_map(self):
        tm = dd(list)
        for tag in self._tags:
            tm[tag.tagtype].append(tag)
        return tm

    @property
    def tags(self):
        return self._tags

    def __str__(self):
        return "`{l}`<{f}:{t}>{tgs}".format(l=self.label, f=self.cfrom, t=self.cto, tgs=self._tags if self._tags else '')

    def tag(self, label, cfrom=None, cto=None, tagtype=None, source=TagInfo.DEFAULT):
        if not cfrom:
            cfrom = self.cfrom
        if not cto:
            cto = self.cto
        tag = TagInfo(label=label, cfrom=cfrom, cto=cto, tagtype=tagtype)
        self._tags.append(tag)

    def to_json(self):
        tagdict = dd(list)
        token_json = {'cfrom': self.cfrom,
                      'cto': self.cto,
                      'label': self.label}
        for k, tags in self.tag_map.items():
            if k == Token.LEMMA:
                token_json[Token.LEMMA] = ', '.join([t.label for t in tags])
                pass
            elif k == Token.POS:
                token_json[Token.POS] = ', '.join([t.label for t in tags])
            elif k == Token.COMMENT:
                token_json[Token.COMMENT] = '\n'.join([t.label for t in tags])
            else:
                tagdict[k if k else 'unsorted'].extend([t.label for t in tags])
        if tagdict:
            token_json['tags'] = dict(tagdict)
        return token_json


class TaggedDoc(object):

    def __init__(self, doc_path, doc_name):
        self.path = FileHelper.abspath(doc_path)
        self.name = doc_name
        self.sents = []
        self.sent_map = {}

    def __len__(self):
        return len(self.sents)

    def __getitem__(self, idx):
        return self.sents[idx]

    def add_sent(self, text, ID):
        sent = TaggedSentence(text, ID=ID)
        self.sents.append(sent)
        self.sent_map[ID] = sent
        return sent

    @staticmethod
    def from_json_file(doc_path, doc_name=None):
        if not os.path.isfile(doc_path):
            raise Exception("Document file could not be found: {}".format(doc_path))
        if not doc_name:
            doc_name = FileHelper.getfilename(doc_path)
        doc = TaggedDoc(doc_path, doc_name)
        with open(doc_path, 'rt') as infile:
            for line in infile:
                j = json.loads(line)
                sent = TaggedSentence.from_json(j)
                doc.sents.append(sent)
                doc.sent_map[sent.ID] = sent
        return doc

    @property
    def sent_path(self):
        return os.path.join(self.path, '{}_sents.txt'.format(self.name))

    @property
    def word_path(self):
        return os.path.join(self.path, '{}_words.txt'.format(self.name))

    @property
    def concept_path(self):
        return os.path.join(self.path, '{}_concepts.txt'.format(self.name))

    @property
    def link_path(self):
        return os.path.join(self.path, '{}_links.txt'.format(self.name))

    @property
    def tag_path(self):
        return os.path.join(self.path, '{}_tags.txt'.format(self.name))

    def read(self):
        ''' Read tagged doc from files (sents, words, concepts, links, tags) '''
        if not os.path.isfile(self.sent_path):
            raise Exception("Document file could not be found {}".format(self.sent_path))
        sent_rows = CSV.read_tsv(self.sent_path)
        for sid, text in sent_rows:
            self.add_sent(text.strip(), ID=sid)
        # Read words if available
        if os.path.isfile(self.word_path):
            sent_words_map = dd(list)
            word_rows = CSV.read_tsv(self.word_path)
            for sid, wid, word, lemma, pos in word_rows:
                sent_words_map[sid].append((word, lemma, pos.strip(), wid))
                # TODO: verify wid?
            # import words
            for sent in self.sents:
                sent_words = sent_words_map[sent.ID]
                sent.import_tokens([x[0] for x in sent_words])
                for ((word, lemma, pos, wid), token) in zip(sent_words, sent.tokens):
                    token.tag(tagtype=Token.POS, label=pos)
                    token.tag(tagtype=Token.LEMMA, label=lemma)
                    token.tag(tagtype='wid', label=wid)
            # only read concepts if words are available
            if os.path.isfile(self.concept_path):
                # read concepts
                concept_rows = CSV.read_tsv(self.concept_path)
                for sid, cid, clemma, tag in concept_rows:
                    self.sent_map[sid].add_concept(cid, clemma, tag.strip())
                # only read concept-word links if words and concepts are available
                link_rows = CSV.read_tsv(self.link_path)
                for sid, cid, wid in link_rows:
                    sent = self.sent_map[sid]
                    wid = int(wid.strip())
                    sent.concept_map[cid].words.append(sent[wid])
        # read sentence level tags
        if os.path.isfile(self.tag_path):
            rows = CSV.read_tsv(self.tag_path)
            for sid, cfrom, cto, label, tagtype in rows:
                self.sent_map[sid].add_tag(label, cfrom, cto, tagtype=tagtype)
        return self

    def export(self, sent_stream, word_stream, concept_stream, link_stream, tag_stream):
        # export to TTL format (multiple text files)
        sent_writer = csv.writer(sent_stream, dialect=STD_DIALECT, quoting=STD_QUOTING)
        word_writer = csv.writer(word_stream, dialect=STD_DIALECT, quoting=STD_QUOTING)
        concept_writer = csv.writer(concept_stream, dialect=STD_DIALECT, quoting=STD_QUOTING)
        link_writer = csv.writer(link_stream, dialect=STD_DIALECT, quoting=STD_QUOTING)
        tag_writer = csv.writer(tag_stream, dialect=STD_DIALECT, quoting=STD_QUOTING)
        for sent in self.sents:
            sent_writer.writerow((sent.ID, sent.text))
            # write words
            for wid, word in enumerate(sent.tokens):
                word_writer.writerow((word.sent.ID, wid, word.surface, word.lemma, word.pos))
            # write concepts & wclinks
            for cid, concept in enumerate(sent.concepts):
                # write concept
                concept_writer.writerow((sent.ID, cid, concept.clemma, concept.tag))
                # write cwlinks
                for word in concept.words:
                    wid = sent.tokens.index(word)
                    link_writer.writerow((sent.ID, cid, wid))
            # write tags
            for tag in sent.tags:
                tag_writer.writerow((sent.ID, tag.cfrom, tag.cto, tag.label, tag.tagtype))

    def write_ttl(self):
        with FileHub(working_dir=self.path, default_mode='w') as output:
            self.export(output[self.sent_path],
                        output[self.word_path],
                        output[self.concept_path],
                        output[self.link_path],
                        output[self.tag_path])


class Concept(object):

    FLAG = 'flag'
    COMMENT = 'comment'
    NOT_MATCHED = 'E'

    def __init__(self, cid, clemma, tag, sent, words=None, comment=None):
        self.cid = cid
        self.clemma = clemma
        self.tag = tag
        self.sent = sent
        self.comment = ''
        if words:
            if type(words) == list:
                self.words = words
            else:
                self.words = list(words)
        else:
            self.words = []
        self.flag = None

    def add_word(self, word):
        self.words.append(word)

    def __repr__(self):
        return '<{t}:"{l}">'.format(l=self.clemma, t=self.tag)

    def __str__(self):
        return '<{t}:"{l}">({ws})'.format(l=self.clemma, t=self.tag, ws=self.words)

    def to_json(self):
        if self.sent:
            # get word idx from sent
            words = [self.sent.tokens.index(w) for w in self.words]
        else:
            words = [w.label for w in self.words]
        cdict = {
            'clemma': self.clemma,
            'tag': self.tag,
            'words': words
        }
        if self.comment:
            cdict[Concept.COMMENT] = self.comment
        if self.flag:
            cdict[Concept.FLAG] = self.flag
        return cdict


class StringBuffer:
    def __init__(self, text=None):
        self.buff = []
        self.__length = 0
        self.append(text)
        self.write = self.append

    def clear(self):
        del self.buff[:]

    def append(self, text):
        if text:
            self.buff.append(text)
            self.__length += len(text)
        return self

    def writeline(self, text):
        return self.append(text).newline()

    def newline(self):
        return self.append('\n')

    def size(self):
        return self.__length

    def __len__(self):
        return self.__length

    def __str__(self):
        return ''.join(self.buff)



# -*- coding: utf-8 -*-

'''
Dao Phay library: A collection of tools for processing Vietnamese text using Python.

Latest version can be found at https://github.com/letuananh/chirptext

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2015, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

# in lower case: aàảãáạăằẳẵắặâầẩẫấậbcdđeèẻẽéẹêềểễếệghiìỉĩíịklmnoòỏõóọôồổỗốộơờởỡớợpqrstuùủũúụưừửữứựvxyỳỷỹýỵ
VIETNAMESE_ALPHABET_ORDER = 'aAàÀảẢãÃáÁạẠăĂằẰẳẲẵẴắẮặẶâÂầẦẩẨẫẪấẤậẬbBcCdDđĐeEèÈẻẺẽẼéÉẹẸêÊềỀểỂễỄếẾệỆgGhHiIìÌỉỈĩĨíÍịỊkKlLmMnNoOòÒỏỎõÕóÓọỌôÔồỒổỔỗỖốỐộỘơƠờỜởỞỡỠớỚợỢpPqQrRsStTuUùÙủỦũŨúÚụỤưƯừỪửỬữỮứỨựỰvVxXyYỳỲỷỶỹỸýÝỵỴ'
VIETNAMESE_SORTING_DICT = dict((x, VIETNAMESE_ALPHABET_ORDER.index(x)) for x in VIETNAMESE_ALPHABET_ORDER)


# ------------------------------------------------------------------------------
# Classes
# ------------------------------------------------------------------------------

class DaoPhay:
    @staticmethod
    def vn_sorted(list_of_strings):
        return sorted(list_of_strings, key=lambda x: [VIETNAMESE_SORTING_DICT[c] for c in x])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Script for testing leutile

Latest version can be found at https://github.com/letuananh/chirptext

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2015, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
import unittest
import logging
from chirptext import TextReport
from chirptext.texttaglib import TagInfo, TaggedSentence, TaggedDoc, Token
from chirptext.deko import txt2mecab

# ------------------------------------------------------------------------------
# Configuration
# ------------------------------------------------------------------------------

TEST_DIR = os.path.dirname(__file__)
TEST_DATA = os.path.join(TEST_DIR, 'data')

BARK_SID = '01047745-v'
GDOG_SID = '02103841-n'


def getLogger():
    return logging.getLogger(__name__)


# ------------------------------------------------------------------------------
# Tests
# ------------------------------------------------------------------------------

class TestBuildTags(unittest.TestCase):

    def test_create_sent(self):
        doc = TaggedDoc(TEST_DATA, 'test')
        # add words
        sent = doc.add_sent('Some happy guard-dogs barked.', 1)
        sent.import_tokens('Some happy guard dogs barked .'.split())
        self.assertEqual(len(sent), 6)
        # sense tagging
        sent.add_concept(0, 'happy', '01148283-a', [sent[1]])
        sent.add_concept(1, 'dog', '02084071-n')
        sent.concept(1).add_word(sent[3])
        sent.tag('bark', BARK_SID, 4)
        sent.tag('guard-dog', GDOG_SID, 2, 3)  # MWE example
        # verification
        wcl = sent.wclinks
        self.assertEqual(wcl[sent[3]][0].clemma, 'dog')
        self.assertEqual(wcl[sent[3]][1].clemma, 'guard-dog')
        self.assertEqual(wcl[sent[3]][1].tag, GDOG_SID)
        self.assertEqual(wcl[sent[4]][0].tag, BARK_SID)
        self.assertTrue(sent.mwe)
        self.assertEqual(sent.mwe[0].tag, GDOG_SID)


class TestTagging(unittest.TestCase):

    def test_taginfo(self):
        print("Test tag info")
        t = TagInfo(1, 4, 'dog', TagInfo.ISF)
        self.assertEqual(t.cfrom, 1)
        self.assertEqual(t.cto, 4)
        self.assertEqual(t.label, 'dog')
        self.assertEqual(t.source, TagInfo.ISF)

        # Source = DEFAULT now
        t2 = TagInfo(0, 5, 'bark')
        self.assertEqual(t2.cfrom, 0)
        self.assertEqual(t2.cto, 5)
        self.assertEqual(t2.label, 'bark')
        self.assertEqual(t2.source, TagInfo.DEFAULT)

    def test_tag_token(self):
        token = Token(0, 4, "Words")
        token.tag("n", tagtype="pos")
        token.tag("plural")
        token.tag("word-token")
        token.tag("word", tagtype="lemma")
        token.tag("Words", tagtype="lemma")
        token.tag("an element of speech or writing", tagtype="comment")
        js_token = token.to_json()
        expected = {"tags": {"unsorted": ["plural", "word-token"]}, "cfrom": 0, "pos": "n", "cto": 4, "label": "Words", "lemma": "word, Words", 'comment': 'an element of speech or writing'}
        self.assertEqual(js_token, expected)

    def test_comment(self):
        sent = TaggedSentence("Dogs bark.")
        sent.import_tokens("Dogs bark .".split())
        sent[0].tag("canine", tagtype="comment")
        sent.tag("02084071-n", "dog", 0)
        sent.concepts[0].comment = 'a member of the genus Canis (probably descended from the common wolf) that has been domesticated by man since prehistoric times'
        expected = {'text': 'Dogs bark.', 'tokens': [{'cto': 4, 'cfrom': 0, 'comment': 'canine', 'label': 'Dogs'}, {'cto': 9, 'cfrom': 5, 'label': 'bark'}, {'cto': 10, 'cfrom': 9, 'label': '.'}], 'concepts': [{'tag': 'dog', 'clemma': '02084071-n', 'comment': 'a member of the genus Canis (probably descended from the common wolf) that has been domesticated by man since prehistoric times', 'words': [0]}]}
        self.assertEqual(expected, sent.to_json())
        self.assertFalse(sent.tags)
        sent.add_tag(GDOG_SID, 0, 4, tagtype='wn30')
        sent.add_tag(BARK_SID, 5, 9, tagtype='wn30')
        for t in sent.tags:
            getLogger().debug("{}: label={} | type = {}".format(t, t.label, t.tagtype))

    def test_tagged_sentences(self):
        sent = TaggedSentence('猫が好きです 。')
        mecab_sent = txt2mecab(sent.text)
        # import tags
        sent.import_tokens(mecab_sent.words)
        for mtoken, token in zip(mecab_sent, sent.tokens):
            if mtoken.reading_hira() != token.label:
                if token.label in ('猫', '好き'):
                    token.tag(mtoken.reading_hira(), tagtype=Token.LEMMA)
                token.tag(mtoken.reading_hira())
                token.tag(mtoken.pos3(), tagtype="pos")
        self.assertEqual(mecab_sent.words, [x.label for x in sent.tokens])
        self.assertEqual(sent.tokens[0].tags[0].label, 'ねこ')
        self.assertEqual(sent.tokens[0].lemma, 'ねこ')
        self.assertEqual(sent.tokens[2].tags[0].label, 'すき')
        self.assertEqual(sent.tokens[3].lemma, '')  # if there is no lemma label => return ''
        self.assertEqual(sent.tokens[2].surface, '好き')
        self.assertFalse(sent.tokens[1].tags)
        self.assertFalse(sent.tokens[3].tags)
        self.assertFalse(sent.tokens[4].tags)
        return sent

    def test_tagged_sent_to_json(self):
        sent = TaggedSentence("女の子は猫が好きです。")
        sent.import_tokens("女 の 子 は 猫 が 好き です 。".split())
        sent[0].tag("おんな", tagtype="lemma")
        sent[2].tag("こ", tagtype="lemma")
        sent[4].tag("ねこ", tagtype="lemma")
        sent[4].tag("Say neh-koh", tagtype="comment")
        sent[4].tag("名詞-一般", tagtype="pos")
        sent[6].tag("すき", tagtype="lemma")
        sent[6].tag("名詞-形容動詞語幹", tagtype="pos")
        sent.tag("女の子", " 10084295-n", 0, 1, 2)
        sent.concepts[0].comment = "若々しい女の人"
        sent.tag("猫", "02121620-n", 4)
        sent.tag("好き", "01292683-a", 6)
        expected = {'tokens': [{'cfrom': 0, 'cto': 1, 'lemma': 'おんな', 'label': '女'}, {'cfrom': 1, 'cto': 2, 'label': 'の'}, {'cfrom': 2, 'cto': 3, 'lemma': 'こ', 'label': '子'}, {'cfrom': 3, 'cto': 4, 'label': 'は'}, {'pos': '名詞-一般', 'cfrom': 4, 'cto': 5, 'lemma': 'ねこ', 'label': '猫', 'comment': 'Say neh-koh'}, {'cfrom': 5, 'cto': 6, 'label': 'が'}, {'pos': '名詞-形容動詞語幹', 'cfrom': 6, 'cto': 8, 'lemma': 'すき', 'label': '好き'}, {'cfrom': 8, 'cto': 10, 'label': 'です'}, {'cfrom': 10, 'cto': 11, 'label': '。'}], 'text': '女の子は猫が好きです。', 'concepts': [{'tag': ' 10084295-n', 'words': [0, 1, 2], 'clemma': '女の子', 'comment': '若々しい女の人'}, {'tag': '02121620-n', 'words': [4], 'clemma': '猫'}, {'tag': '01292683-a', 'words': [6], 'clemma': '好き'}]}
        actual = sent.to_json()
        self.assertEqual(expected, actual)

    def test_json_tagged_sent(self):
        raw = {'tokens': [{'cfrom': 0, 'cto': 1, 'lemma': 'おんな', 'label': '女'}, {'cfrom': 1, 'cto': 2, 'label': 'の'}, {'cfrom': 2, 'cto': 3, 'lemma': 'こ', 'label': '子'}, {'cfrom': 3, 'cto': 4, 'label': 'は'}, {'pos': '名詞-一般', 'cfrom': 4, 'cto': 5, 'lemma': 'ねこ', 'label': '猫', 'comment': 'Say neh-koh'}, {'cfrom': 5, 'cto': 6, 'label': 'が'}, {'pos': '名詞-形容動詞語幹', 'cfrom': 6, 'cto': 8, 'lemma': 'すき', 'label': '好き'}, {'cfrom': 8, 'cto': 10, 'label': 'です'}, {'cfrom': 10, 'cto': 11, 'label': '。'}], 'text': '女の子は猫が好きです。', 'concepts': [{'tag': ' 10084295-n', 'words': [0, 1, 2], 'clemma': '女の子', 'comment': '若々しい女の人', 'flag': 'G'}, {'tag': '02121620-n', 'words': [4], 'clemma': '猫'}, {'tag': '01292683-a', 'words': [6], 'clemma': '好き'}]}
        # json => TaggedSentence => json
        sent = TaggedSentence.from_json(raw)
        sent_json = sent.to_json()
        self.assertEqual(raw, sent_json)
        self.assertEqual(sent_json['concepts'][0]['flag'], 'G')
        pass

    def test_tagging_erg_sent(self):
        txt = '''In this way I am no doubt indirectly responsible for Dr. Grimesby Roylott's death, and I cannot say that it is likely to weigh very heavily upon my conscience."'''
        words = ['in', 'this', 'way', 'i', 'am', 'no', 'doubt', 'indirectly', 'responsible', 'for', 'dr.', 'Grimesby', 'Roylott', "'s", 'death', ',', 'and', 'i', 'can', 'not', 'say', 'that', 'it', 'is', 'likely', 'to', 'weigh', 'very', 'heavily', 'upon', 'my', 'conscience', '.', '"']
        s = TaggedSentence(txt)
        s.import_tokens(words)
        self.assertEqual(words, [x.label for x in s.tokens])

    def test_tagged_doc(self):
        doc = TaggedDoc(TEST_DATA, 'test')
        doc.read()
        ts_count = [(len(s), len(s.concepts)) for s in doc]
        self.assertEqual(len(doc), 3)  # 3 sents
        self.assertEqual(ts_count, [(29, 15), (28, 13), (34, 23)])

    def test_cwl(self):
        doc = TaggedDoc(TEST_DATA, 'test').read()
        sent = doc[0]
        # ensure that words and concepts are linked properly
        w = doc[0][-4]
        c = sent.wclinks[w][0]
        tag = w.tag_map['pos'][0]
        self.assertEqual(tag.label, 'WP')
        self.assertEqual(w.pos, 'WP')  # shortcut for POS
        self.assertEqual(c.words[0].tag_map['pos'][0].label, 'WP')
        tag.label = 'x'
        self.assertEqual(c.words[0].tag_map['pos'][0].label, 'x')
        # test sentence level tagging
        getLogger().debug([str(t) for t in sent.tags])

    def test_multiple_tags(self):
        doc = TaggedDoc(TEST_DATA, 'test').read()
        mw_ms = [(len(s.mwe), len(s.msw)) for s in doc]
        self.assertEqual(mw_ms, [(2, 2), (0, 0), (3, 2)])

    def test_recover_surface_string(self):
        s = TaggedSentence('''a religious sect founded in the United States in 1966; based on Vedic scriptures; groups engage in joyful chanting of `Hare Krishna' and other mantras based on the name of the Hindu god Krishna; devotees usually wear saffron robes and practice vegetarianism and celibacy''')
        tokens = ['a', 'religious', 'sect', 'founded', 'in', 'the', 'United', 'States', 'in', '1966', ';', 'based', 'on', 'Vedic', 'scriptures', ';', 'groups', 'engage', 'in', 'joyful', 'chanting', 'of', 'Hare', 'Krishna', 'and', 'other', 'mantras', 'based', 'on', 'the', 'name', 'of', 'the', 'Hindu', 'god', 'Krishna', ';', 'devotees', 'usually', 'wear', 'saffron', 'robes', 'and', 'practice', 'vegetarianism', 'and', 'celibacy']
        s.import_tokens(tokens)
        cfrom = min(x.cfrom for x in s.tokens)
        cto = max(x.cto for x in s.tokens)
        self.assertEqual(s.text, s.text[cfrom:cto])

    def test_export_to_streams(self):
        concepts = TextReport.string()
        links = TextReport.string()
        sents = TextReport.string()
        tags = TextReport.string()
        words = TextReport.string()
        doc = TaggedDoc(TEST_DATA, 'manual')
        # create sents in doc
        raws = ("三毛猫がすきです。", "雨が降る。", "女の子はケーキを食べる。")
        for sid, r in enumerate(raws):
            msent = txt2mecab(r)
            tsent = doc.add_sent(msent.surface, sid)
            tsent.import_tokens(msent.words)
            # pos tagging
            for mtk, tk in zip(msent, tsent):
                tk.tag(mtk.pos3(), tagtype=Token.POS, source=TagInfo.MECAB)
                tk.tag(mtk.reading_hira(), tagtype=Token.LEMMA, source=TagInfo.MECAB)
        # sense tagging
        doc[0].tag("三毛猫", "wiki.ja:三毛猫", 0, 1, 2)
        doc[1].tag("降る", "02756821-v", 2)
        doc[2].tag("女の子", "10084295-n", 0)
        doc[2].tag("食べる", "01166351-v", 4)
        # tags
        doc[0].add_tag("WIKI", 0, 3, tagtype="SRC")
        doc[0].add_tag("https://ja.wikipedia.org/wiki/三毛猫", 0, 3, tagtype="URL")
        doc[2].add_tag("WIKI", 0, 3, tagtype="SRC")
        doc[2].add_tag("https://ja.wikipedia.org/wiki/少女", 0, 3, tagtype="URL")
        # export doc
        doc.export(sents.file, words.file, concepts.file, links.file, tags.file)
        getLogger().debug("sents\n{}".format(sents.content()))
        getLogger().debug("words\n{}".format(words.content()))
        getLogger().debug("concepts\n{}".format(concepts.content()))
        getLogger().debug("links\n{}".format(links.content()))
        getLogger().debug("tags\n{}".format(tags.content()))
        self.assertTrue(sents.content())
        self.assertTrue(words.content())
        self.assertTrue(concepts.content())
        self.assertTrue(links.content())
        self.assertTrue(tags.content())


# ------------------------------------------------------------------------------
# Main
# ------------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

'''
Readers for popular formats

Latest version can be found at https://github.com/letuananh/chirptext

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2017, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import csv
import logging


# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

def getLogger():
    return logging.getLogger()


# -------------------------------------------------------------------------------
# Functions
# -------------------------------------------------------------------------------

class CSV(object):

    QUOTE_MINIMAL = csv.QUOTE_MINIMAL
    QUOTE_NONE = csv.QUOTE_NONE
    QUOTE_ALL = csv.QUOTE_ALL

    @staticmethod
    def read(file_name, dialect=None, header=False, *args, **kwargs):
        with open(file_name, newline='') as csvfile:
            if not dialect:
                # auto detect
                dialect = csv.Sniffer().sniff(csvfile.read(1024))
                csvfile.seek(0)
            if not header:
                reader = csv.reader(csvfile, dialect=dialect, *args, **kwargs)
                return list(reader)
            else:
                reader = csv.DictReader(csvfile, dialect=dialect, *args, **kwargs)
                return list(reader)

    @staticmethod
    def read_tsv(file_name, *args, **kwargs):
        return CSV.read(file_name, dialect='excel-tab', *args, **kwargs)

    @staticmethod
    def write(file_name, rows, dialect='excel', header=None, quoting=csv.QUOTE_ALL, extrasaction='ignore'):
        ''' Write rows data to a CSV file (with or without header) '''
        if not quoting:
            quoting = csv.QUOTE_MINIMAL
        with open(file_name, 'w') as csvfile:
            if not header:
                writer = csv.writer(csvfile, dialect=dialect, quoting=quoting)
                for row in rows:
                    writer.writerow(row)
            else:
                writer = csv.DictWriter(csvfile, dialect=dialect, fieldnames=header, quoting=quoting, extrasaction=extrasaction)
                writer.writeheader()
                for row in rows:
                    writer.writerow(row)

    @staticmethod
    def write_tsv(file_name, rows, quoting=csv.QUOTE_ALL, **kwargs):
        CSV.write(file_name, rows, dialect='excel-tab', quoting=quoting, **kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Script for testing arsenal lib

Latest version can be found at https://github.com/letuananh/chirptext

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2017, Le Tuan Anh <tuananh.ke@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
import logging
import unittest

from chirptext.leutile import LOREM_IPSUM
from chirptext import JiCache

# ----------------------------------------------------------------------
# Configuration
# ----------------------------------------------------------------------

TEST_DIR = os.path.dirname(__file__)
TEST_CACHE = os.path.join(TEST_DIR, 'data', 'test_cache.db')


def getLogger():
    return logging.getLogger(__name__)


########################################################################

class TestMain(unittest.TestCase):

    def try_cache(self, cache, key, value):
        if key in cache:
            cache.delete_blob(key)
        cache.insert_string(key, value)
        self.assertIn(key, cache)
        self.assertEqual(cache.retrieve_string(key), value)

    def test_empty_cache(self):
        TEST_CACHE2 = os.path.join(TEST_DIR, 'test_cache2.db')
        JiCache(TEST_CACHE2)
        self.assertFalse(os.path.exists(TEST_CACHE2))

    def test_arsenal(self):
        print("Test arsenal module")
        cache = JiCache(TEST_CACHE)
        # test insert blob
        self.try_cache(cache, 'lorem', LOREM_IPSUM)
        self.try_cache(cache, 1, 'Key is a number')
        self.try_cache(cache, None, 'None key')


########################################################################

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Chirp Net - Web utilities

Latest version can be found at https://github.com/letuananh/chirptext

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2017, Le Tuan Anh <tuananh.ke@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
from io import BytesIO
import gzip
import logging
import json
from urllib.request import Request, urlopen
from urllib.error import URLError
from urllib.parse import urlparse, urlunparse, parse_qs, urlencode, quote

from chirptext.arsenal import JiCache


# ----------------------------------------------------------------------
# Configuration
# ----------------------------------------------------------------------

def getLogger():
    return logging.getLogger(__name__)


# ----------------------------------------------------------------------
# Functions
# ----------------------------------------------------------------------

class SmartURL(object):
    ''' Smart URL supports URL manipulation '''

    def __init__(self, raw_url, quoted=False):
        self.raw = raw_url
        self.p = None
        self.scheme = None
        self.netloc = None
        self.opath = None
        self.path = None
        self.params = None
        self.query = None
        self.fragment = None
        if self.raw:
            self.process(quoted)

    def get_filename(self):
        return os.path.basename(self.p.path)

    def get_filename_noext(self):
        return os.path.splitext(self.get_filename())[0]

    def get_file_ext(self):
        return os.path.splitext(self.get_filename())[1]

    def process(self, quoted=False):
        ''' Parse an URL '''
        self.p = urlparse(self.raw)
        self.scheme = self.p.scheme
        self.netloc = self.p.netloc
        self.opath = self.p.path if not quoted else quote(self.p.path)
        self.path = [x for x in self.opath.split('/') if x]
        self.params = self.p.params
        self.query = parse_qs(self.p.query, keep_blank_values=True)
        self.fragment = self.p.fragment

    def __str__(self):
        flatd = [(k, v[0] if len(v) == 1 else v) for k, v in self.query.items()]
        return urlunparse((self.scheme, self.netloc, '/'.join(self.path), self.params, urlencode(flatd), self.fragment))


class WebHelper(object):
    ''' a wget like utility for Python '''

    def __init__(self, cache=None):
        if cache is None or isinstance(cache, JiCache):
            self.cache = cache
        else:
            self.cache = JiCache(location=str(cache))

    @staticmethod
    def encode_url(url):
        return str(SmartURL(url, quoted=True))

    def fetch(self, url, encoding=None, force_refetch=False, nocache=False, quiet=True):
        ''' Fetch a HTML file as binary'''
        try:
            if not force_refetch and self.cache is not None and url in self.cache:
                # try to look for content in cache
                logging.debug('Retrieving content from cache for {}'.format(url))
                return self.cache.retrieve_blob(url, encoding)
            encoded_url = WebHelper.encode_url(url)
            req = Request(encoded_url, headers={'User-Agent': 'Mozilla/5.0'})
            # support gzip
            req.add_header('Accept-encoding', 'gzip, deflate')
            # Open URL
            getLogger().info("Fetching: {url} |".format(url=url))
            response = urlopen(req)
            content = response.read()
            # unzip if required
            if 'Content-Encoding' in response.info() and response.info().get('Content-Encoding') == 'gzip':
                # unzip
                with gzip.open(BytesIO(content)) as gzfile:
                    content = gzfile.read()
            # update cache if required
            if self.cache is not None and not nocache:
                if url not in self.cache:
                    self.cache.insert_blob(url, content)
            return content.decode(encoding) if content and encoding else content
        except URLError as e:
            if hasattr(e, 'reason'):
                getLogger().exception('We failed to reach a server. Reason: {}'.format(e.reason))
            elif hasattr(e, 'code'):
                getLogger().exception('The server couldn\'t fulfill the request. Error code: {}'.format(e.code))
            else:
                # Other exception ...
                getLogger().exception("Fetching error")
            if not quiet:
                raise
        return None

    def fetch_json(self, *args, **kwargs):
        output = self.fetch(*args, **kwargs)
        if output is not None:
            return json.loads(output)
        else:
            return None

    def download(self, url, path, force_refetch=False, nocache=False):
        ''' Download a file at $url and save it to $path
        '''
        # Enable cache
        if os.path.isfile(path):
            getLogger().info("File exists, download task skipped -> {path}".format(path=path))
            return True
        try:
            # Open URL
            getLogger().info("Downloading: {url} -> {path}".format(url=url, path=path))
            response = self.fetch(url, force_refetch=force_refetch, nocache=nocache)
            if response is not None:
                # Download file
                local_file = open(path, "wb")
                local_file.write(response)
                local_file.close()
                # Finished
                return True
            else:
                return False
        except Exception as e:
            if hasattr(e, 'reason'):
                getLogger().exception('We failed to reach a server. Reason: %s' % (e.reason,))
            elif hasattr(e, 'code'):
                getLogger().exception("The server couldn't fulfill the request. Error code: {code}".format(code=e.code))
            else:
                # everything is fine
                getLogger().exception("Unknown error: %s" % (e,))
        return False



# -*- coding: utf-8 -*-

'''
Chirp Text - Minimalist Text Processing Library

Latest version can be found at https://github.com/letuananh/chirptext

References:
    Python documentation:
        https://docs.python.org/
    PEP 0008 - Style Guide for Python Code
        https://www.python.org/dev/peps/pep-0008/
    PEP 0257 - Python Docstring Conventions:
        https://www.python.org/dev/peps/pep-0257/

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2012-2017, Le Tuan Anh <tuananh.ke@gmail.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

__author__ = "Le Tuan Anh"
__email__ = "tuananh.ke@gmail.com"
__copyright__ = "Copyright 2017, chirptext"
__credits__ = []
__license__ = "MIT License"
__description__ = "ChirpText is a collection of text processing tools for Python."
__url__ = "https://github.com/letuananh/chirptext"
__maintainer__ = "Le Tuan Anh"
__version_major__ = "0.1"
__version__ = "{}a4".format(__version_major__)
__version_long__ = "{} - Alpha".format(__version_major__)
__status__ = "Prototype"

########################################################################

from .leutile import Timer, Counter, uniquify, header, confirm
from .leutile import FileHub, TextReport, Table, FileHelper, ConfigFile
from .arsenal import JiCache
from .chirpnet import SmartURL, WebHelper

__all__ = ["uniquify", "header", "confirm",
           "Timer", "Counter", "Table",
           "TextReport", "FileHub", "FileHelper", "ConfigFile",
           "JiCache", "SmartURL", "WebHelper"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Test script for deko module

Latest version can be found at https://github.com/letuananh/chirptext

@author: Le Tuan Anh <tuananh.ke@gmail.com>
@license: MIT
'''

# Copyright (c) 2017, Le Tuan Anh <tuananh.ke@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

########################################################################

import os
import unittest
import logging

from chirptext.deko import wakati, tokenize, analyse, txt2mecab, tokenize_sent, DekoText

# -------------------------------------------------------------------------------
# Configuration
# -------------------------------------------------------------------------------

TEST_DIR = os.path.dirname(os.path.realpath(__file__))
txt = '雨が降る。'
txt2 = '猫が好きです。\n犬も好きです。'
txt3 = '猫が好きです。\n犬も好きです。\n鳥は'


def getLogger():
    return logging.getLogger(__name__)


# -------------------------------------------------------------------------------
# Data Structures
# -------------------------------------------------------------------------------

class TestMainApp(unittest.TestCase):

    def test_mecab(self):
        tokens = txt2mecab(txt)
        self.assertTrue(tokens[-1].is_eos)

    def test_wakati(self):
        tks = wakati(txt)
        self.assertEqual(tks, '雨 が 降る 。 \n')

    def test_deko(self):
        tokenized = tokenize(txt)
        self.assertEqual(tokenized, ['雨', 'が', '降る', '。'])

    def test_analyse2(self):
        sents = DekoText.parse(txt)
        self.assertEqual(len(sents), 1)
        self.assertEqual(len(sents[0]), 5)
        self.assertEqual(str(sents[0]), '雨 が 降る 。')
        # 2 sentences
        sents = DekoText.parse(txt2)
        self.assertEqual(len(sents), 2)
        self.assertEqual(len(sents[0]), 6)
        self.assertEqual(len(sents[1]), 6)
        self.assertEqual(sents[0].words, ['猫', 'が', '好き', 'です', '。'])
        self.assertEqual(str(sents[1]), '犬 も 好き です 。')
        # 2 sentences
        sents = DekoText.parse(txt2, splitlines=False)
        self.assertEqual(len(sents), 2)
        self.assertEqual(len(sents[0]), 5)
        self.assertEqual(len(sents[1]), 5)
        self.assertEqual(str(sents[0]), '猫 が 好き です 。')
        self.assertEqual(str(sents[1]), '犬 も 好き です 。')

    def test_analyse(self):
        sents = analyse(txt2, format='txt')
        self.assertEqual(sents, '猫 が 好き です 。\n犬 も 好き です 。')
        # test sent tokenizing using MeCab
        tokens = txt2mecab(txt2)
        sents = tokenize_sent(tokens)
        # using analyse function
        sents = analyse(txt2, splitlines=False, format='txt').split('\n')
        self.assertEqual(sents, ['猫 が 好き です 。', '犬 も 好き です 。'])
        # to html
        h = analyse(txt, format='html')
        self.assertEqual(h, '<ruby><rb>雨</rb><rt>あめ</rt></ruby> が <ruby><rb>降る</rb><rt>ふる</rt></ruby>。 ')
        # to csv
        c = analyse(txt, format='csv')
        e = '''雨	名詞	一般	*	*	*	*	雨	アメ	アメ
が	助詞	格助詞	一般	*	*	*	が	ガ	ガ
降る	動詞	自立	*	*	五段・ラ行	基本形	降る	フル	フル
。	記号	句点	*	*	*	*	。	。	。
EOS									
'''
        self.assertEqual(c, e)

    def test_pos(self):
        sent = txt2mecab(txt)
        self.assertTrue(sent[-1].is_eos)
        poses = [tk.pos3() for tk in sent if not tk.is_eos]
        self.assertEqual(poses, ['名詞-一般', '助詞-格助詞-一般', '動詞-自立', '記号-句点'])
        for tk in sent:
            getLogger().debug(tk.pos3())

    def test_not_split(self):
        doc = DekoText.parse(txt3, splitlines=False)
        docx = DekoText.parse(txt3, splitlines=True)
        self.assertEqual([x.surface for x in doc], [x.surface for x in docx])
        getLogger().debug(doc)


# -------------------------------------------------------------------------------
# Main
# -------------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main()
